@@ -1,4 +1,5 @@
 """Module describing the planemo ``mulled_init`` command."""
+
 import click
 
 from planemo import options
@@ -6,7 +7,7 @@ from planemo.cli import command_function
 from planemo.mulled import build_involucro_context
 
 
-@click.command('mulled_init')
+@click.command("mulled_init")
 @options.mulled_options()
 @command_function
 def cli(ctx, **kwds):
